@@ -1,8 +1,8 @@
 /*
  * LUKS - Linux Unified Key Setup v2
  *
- * Copyright (C) 2015-2018, Red Hat, Inc. All rights reserved.
- * Copyright (C) 2015-2018, Milan Broz. All rights reserved.
+ * Copyright (C) 2015-2019 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2015-2019 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,34 +26,37 @@
 /*
  * Helper functions
  */
-json_object *parse_json_len(const char *json_area, int length, int *end_offset)
+json_object *parse_json_len(struct crypt_device *cd, const char *json_area,
+			    uint64_t max_length, int *json_len)
 {
 	json_object *jobj;
 	struct json_tokener *jtok;
 
-	if (!json_area || length <= 0)
+	 /* INT32_MAX is internal (json-c) json_tokener_parse_ex() limit */
+	if (!json_area || max_length > INT32_MAX)
 		return NULL;
 
 	jtok = json_tokener_new();
 	if (!jtok) {
-		log_dbg("ERROR: Failed to init json tokener");
+		log_dbg(cd, "ERROR: Failed to init json tokener");
 		return NULL;
 	}
 
-	jobj = json_tokener_parse_ex(jtok, json_area, length);
+	jobj = json_tokener_parse_ex(jtok, json_area, max_length);
 	if (!jobj)
-		log_dbg("ERROR: Failed to parse json data (%d): %s",
+		log_dbg(cd, "ERROR: Failed to parse json data (%d): %s",
 			json_tokener_get_error(jtok),
 			json_tokener_error_desc(json_tokener_get_error(jtok)));
 	else
-		*end_offset = jtok->char_offset;
+		*json_len = jtok->char_offset;
 
 	json_tokener_free(jtok);
 
 	return jobj;
 }
 
-static void log_dbg_checksum(const uint8_t *csum, const char *csum_alg, const char *info)
+static void log_dbg_checksum(struct crypt_device *cd,
+			     const uint8_t *csum, const char *csum_alg, const char *info)
 {
 	char csum_txt[2*LUKS2_CHECKSUM_L+1];
 	int i;
@@ -62,7 +65,7 @@ static void log_dbg_checksum(const uint8_t *csum, const char *csum_alg, const ch
 		snprintf(&csum_txt[i*2], 3, "%02hhx", (const char)csum[i]);
 	csum_txt[i*2+1] = '\0'; /* Just to be safe, sprintf should write \0 there. */
 
-	log_dbg("Checksum:%s (%s)", &csum_txt[0], info);
+	log_dbg(cd, "Checksum:%s (%s)", &csum_txt[0], info);
 }
 
 /*
@@ -97,7 +100,8 @@ static int hdr_checksum_calculate(const char *alg, struct luks2_hdr_disk *hdr_di
 /*
  * Compare hash (checksum) of on-disk and in-memory header.
  */
-static int hdr_checksum_check(const char *alg, struct luks2_hdr_disk *hdr_disk,
+static int hdr_checksum_check(struct crypt_device *cd,
+			      const char *alg, struct luks2_hdr_disk *hdr_disk,
 			      const char *json_area, size_t json_len)
 {
 	struct luks2_hdr_disk hdr_tmp;
@@ -115,8 +119,8 @@ static int hdr_checksum_check(const char *alg, struct luks2_hdr_disk *hdr_disk,
 	if (r < 0)
 		return r;
 
-	log_dbg_checksum(hdr_disk->csum, alg, "on-disk");
-	log_dbg_checksum(hdr_tmp.csum, alg, "in-memory");
+	log_dbg_checksum(cd, hdr_disk->csum, alg, "on-disk");
+	log_dbg_checksum(cd, hdr_tmp.csum, alg, "in-memory");
 
 	if (memcmp(hdr_tmp.csum, hdr_disk->csum, (size_t)hash_size))
 		return -EINVAL;
@@ -186,7 +190,8 @@ static void hdr_to_disk(struct luks2_hdr *hdr,
 /*
  * Sanity checks before checksum is validated
  */
-static int hdr_disk_sanity_check_pre(struct luks2_hdr_disk *hdr,
+static int hdr_disk_sanity_check_pre(struct crypt_device *cd,
+				     struct luks2_hdr_disk *hdr,
 				     size_t *hdr_json_size, int secondary,
 				     uint64_t offset)
 {
@@ -194,19 +199,25 @@ static int hdr_disk_sanity_check_pre(struct luks2_hdr_disk *hdr,
 		return -EINVAL;
 
 	if (be16_to_cpu(hdr->version) != 2) {
-		log_dbg("Unsupported LUKS2 header version %u.", be16_to_cpu(hdr->version));
+		log_dbg(cd, "Unsupported LUKS2 header version %u.", be16_to_cpu(hdr->version));
 		return -EINVAL;
 	}
 
 	if (offset != be64_to_cpu(hdr->hdr_offset)) {
-		log_dbg("LUKS2 offset 0x%04x on device differs to expected offset 0x%04x.",
+		log_dbg(cd, "LUKS2 offset 0x%04x on device differs to expected offset 0x%04x.",
 			(unsigned)be64_to_cpu(hdr->hdr_offset), (unsigned)offset);
 		return -EINVAL;
 	}
 
+	if (secondary && (offset != be64_to_cpu(hdr->hdr_size))) {
+		log_dbg(cd, "LUKS2 offset 0x%04x in secondary header doesn't match size 0x%04x.",
+			(unsigned)offset, (unsigned)be64_to_cpu(hdr->hdr_size));
+		return -EINVAL;
+	}
+
 	/* FIXME: sanity check checksum alg. */
 
-	log_dbg("LUKS2 header version %u of size %u bytes, checksum %s.",
+	log_dbg(cd, "LUKS2 header version %u of size %u bytes, checksum %s.",
 		(unsigned)be16_to_cpu(hdr->version), (unsigned)be64_to_cpu(hdr->hdr_size),
 		hdr->checksum_alg);
 
@@ -217,16 +228,17 @@ static int hdr_disk_sanity_check_pre(struct luks2_hdr_disk *hdr,
 /*
  * Read LUKS2 header from disk at specific offset.
  */
-static int hdr_read_disk(struct device *device, struct luks2_hdr_disk *hdr_disk,
+static int hdr_read_disk(struct crypt_device *cd,
+			 struct device *device, struct luks2_hdr_disk *hdr_disk,
 			 char **json_area, uint64_t offset, int secondary)
 {
 	size_t hdr_json_size = 0;
 	int devfd = -1, r;
 
-	log_dbg("Trying to read %s LUKS2 header at offset 0x%" PRIx64 ".",
+	log_dbg(cd, "Trying to read %s LUKS2 header at offset 0x%" PRIx64 ".",
 		secondary ? "secondary" : "primary", offset);
 
-	devfd = device_open_locked(device, O_RDONLY);
+	devfd = device_open_locked(cd, device, O_RDONLY);
 	if (devfd < 0)
 		return devfd == -1 ? -EIO : devfd;
 
@@ -234,14 +246,14 @@ static int hdr_read_disk(struct device *device, struct luks2_hdr_disk *hdr_disk,
 	 * Read binary header and run sanity check before reading
 	 * JSON area and validating checksum.
 	 */
-	if (read_lseek_blockwise(devfd, device_block_size(device),
+	if (read_lseek_blockwise(devfd, device_block_size(cd, device),
 				 device_alignment(device), hdr_disk,
 				 LUKS2_HDR_BIN_LEN, offset) != LUKS2_HDR_BIN_LEN) {
 		close(devfd);
 		return -EIO;
 	}
 
-	r = hdr_disk_sanity_check_pre(hdr_disk, &hdr_json_size, secondary, offset);
+	r = hdr_disk_sanity_check_pre(cd, hdr_disk, &hdr_json_size, secondary, offset);
 	if (r < 0) {
 		close(devfd);
 		return r;
@@ -256,7 +268,7 @@ static int hdr_read_disk(struct device *device, struct luks2_hdr_disk *hdr_disk,
 		return -ENOMEM;
 	}
 
-	if (read_lseek_blockwise(devfd, device_block_size(device),
+	if (read_lseek_blockwise(devfd, device_block_size(cd, device),
 				 device_alignment(device), *json_area, hdr_json_size,
 				 offset + LUKS2_HDR_BIN_LEN) != (ssize_t)hdr_json_size) {
 		close(devfd);
@@ -270,9 +282,9 @@ static int hdr_read_disk(struct device *device, struct luks2_hdr_disk *hdr_disk,
 	/*
 	 * Calculate and validate checksum and zero it afterwards.
 	 */
-	if (hdr_checksum_check(hdr_disk->checksum_alg, hdr_disk,
+	if (hdr_checksum_check(cd, hdr_disk->checksum_alg, hdr_disk,
 				*json_area, hdr_json_size)) {
-		log_dbg("LUKS2 header checksum error (offset %" PRIu64 ").", offset);
+		log_dbg(cd, "LUKS2 header checksum error (offset %" PRIu64 ").", offset);
 		r = -EINVAL;
 	}
 	memset(hdr_disk->csum, 0, LUKS2_CHECKSUM_L);
@@ -283,20 +295,21 @@ static int hdr_read_disk(struct device *device, struct luks2_hdr_disk *hdr_disk,
 /*
  * Write LUKS2 header to disk at specific offset.
  */
-static int hdr_write_disk(struct device *device, struct luks2_hdr *hdr,
-		   const char *json_area, int secondary)
+static int hdr_write_disk(struct crypt_device *cd,
+			  struct device *device, struct luks2_hdr *hdr,
+			  const char *json_area, int secondary)
 {
 	struct luks2_hdr_disk hdr_disk;
 	uint64_t offset = secondary ? hdr->hdr_size : 0;
 	size_t hdr_json_len;
 	int devfd = -1, r;
 
-	log_dbg("Trying to write LUKS2 header (%zu bytes) at offset %" PRIu64 ".",
+	log_dbg(cd, "Trying to write LUKS2 header (%zu bytes) at offset %" PRIu64 ".",
 		hdr->hdr_size, offset);
 
 	/* FIXME: read-only device silent fail? */
 
-	devfd = device_open_locked(device, O_RDWR);
+	devfd = device_open_locked(cd, device, O_RDWR);
 	if (devfd < 0)
 		return devfd == -1 ? -EINVAL : devfd;
 
@@ -307,7 +320,7 @@ static int hdr_write_disk(struct device *device, struct luks2_hdr *hdr,
 	/*
 	 * Write header without checksum but with proper seqid.
 	 */
-	if (write_lseek_blockwise(devfd, device_block_size(device),
+	if (write_lseek_blockwise(devfd, device_block_size(cd, device),
 				  device_alignment(device), (char *)&hdr_disk,
 				  LUKS2_HDR_BIN_LEN, offset) < (ssize_t)LUKS2_HDR_BIN_LEN) {
 		close(devfd);
@@ -317,7 +330,7 @@ static int hdr_write_disk(struct device *device, struct luks2_hdr *hdr,
 	/*
 	 * Write json area.
 	 */
-	if (write_lseek_blockwise(devfd, device_block_size(device),
+	if (write_lseek_blockwise(devfd, device_block_size(cd, device),
 				  device_alignment(device),
 				  CONST_CAST(char*)json_area, hdr_json_len,
 				  LUKS2_HDR_BIN_LEN + offset) < (ssize_t)hdr_json_len) {
@@ -334,44 +347,18 @@ static int hdr_write_disk(struct device *device, struct luks2_hdr *hdr,
 		close(devfd);
 		return r;
 	}
-	log_dbg_checksum(hdr_disk.csum, hdr_disk.checksum_alg, "in-memory");
+	log_dbg_checksum(cd, hdr_disk.csum, hdr_disk.checksum_alg, "in-memory");
 
-	if (write_lseek_blockwise(devfd, device_block_size(device),
+	if (write_lseek_blockwise(devfd, device_block_size(cd, device),
 				  device_alignment(device), (char *)&hdr_disk,
 				  LUKS2_HDR_BIN_LEN, offset) < (ssize_t)LUKS2_HDR_BIN_LEN)
 		r = -EIO;
 
-	device_sync(device, devfd);
+	device_sync(cd, device, devfd);
 	close(devfd);
 	return r;
 }
 
-static int LUKS2_check_device_size(struct crypt_device *cd, struct device *device,
-				   uint64_t hdr_size, int falloc)
-{
-	uint64_t dev_size;
-
-	if (device_size(device, &dev_size)) {
-		log_dbg("Cannot get device size for device %s.", device_path(device));
-		return -EIO;
-	}
-
-	log_dbg("Device size %" PRIu64 ", header size %"
-		PRIu64 ".", dev_size, hdr_size);
-
-	if (hdr_size > dev_size) {
-		/* If it is header file, increase its size */
-		if (falloc && !device_fallocate(device, hdr_size))
-			return 0;
-
-		log_err(cd, _("Device %s is too small. (LUKS2 requires at least %" PRIu64 " bytes.)"),
-			device_path(device), hdr_size);
-		return -EINVAL;
-	}
-
-	return 0;
-}
-
 /*
  * Convert in-memory LUKS2 header and write it to disk.
  * This will increase sequence id, write both header copies and calculate checksum.
@@ -384,16 +371,11 @@ int LUKS2_disk_hdr_write(struct crypt_device *cd, struct luks2_hdr *hdr, struct
 	int r;
 
 	if (hdr->version != 2) {
-		log_dbg("Unsupported LUKS2 header version (%u).", hdr->version);
+		log_dbg(cd, "Unsupported LUKS2 header version (%u).", hdr->version);
 		return -EINVAL;
 	}
 
-	if (hdr->hdr_size != LUKS2_HDR_16K_LEN) {
-		log_dbg("Unsupported LUKS2 header size (%zu).", hdr->hdr_size);
-		return -EINVAL;
-	}
-
-	r = LUKS2_check_device_size(cd, crypt_metadata_device(cd), LUKS2_hdr_and_areas_size(hdr->jobj), 1);
+	r = device_check_size(cd, crypt_metadata_device(cd), LUKS2_hdr_and_areas_size(hdr->jobj), 1);
 	if (r)
 		return r;
 
@@ -412,12 +394,12 @@ int LUKS2_disk_hdr_write(struct crypt_device *cd, struct luks2_hdr *hdr, struct
 	json_text = json_object_to_json_string_ext(hdr->jobj,
 			JSON_C_TO_STRING_PLAIN | JSON_C_TO_STRING_NOSLASHESCAPE);
 	if (!json_text || !*json_text) {
-		log_dbg("Cannot parse JSON object to text representation.");
+		log_dbg(cd, "Cannot parse JSON object to text representation.");
 		free(json_area);
 		return -ENOMEM;
 	}
 	if (strlen(json_text) > (json_area_len - 1)) {
-		log_dbg("JSON is too large (%zu > %zu).", strlen(json_text), json_area_len);
+		log_dbg(cd, "JSON is too large (%zu > %zu).", strlen(json_text), json_area_len);
 		free(json_area);
 		return -EINVAL;
 	}
@@ -434,95 +416,96 @@ int LUKS2_disk_hdr_write(struct crypt_device *cd, struct luks2_hdr *hdr, struct
 	}
 
 	/* Write primary and secondary header */
-	r = hdr_write_disk(device, hdr, json_area, 0);
+	r = hdr_write_disk(cd, device, hdr, json_area, 0);
 	if (!r)
-		r = hdr_write_disk(device, hdr, json_area, 1);
+		r = hdr_write_disk(cd, device, hdr, json_area, 1);
 
 	if (r)
-		log_dbg("LUKS2 header write failed (%d).", r);
+		log_dbg(cd, "LUKS2 header write failed (%d).", r);
 
-	device_write_unlock(device);
+	device_write_unlock(cd, device);
 
 	/* FIXME: try recovery here? */
 
 	free(json_area);
 	return r;
 }
-
-static int validate_json_area(const char *json_area, int start, int length)
+static int validate_json_area(struct crypt_device *cd, const char *json_area,
+			      uint64_t json_len, uint64_t max_length)
 {
 	char c;
 
 	/* Enforce there are no needless opening bytes */
 	if (*json_area != '{') {
-		log_dbg("ERROR: Opening character must be left curly bracket: '{'.");
+		log_dbg(cd, "ERROR: Opening character must be left curly bracket: '{'.");
 		return -EINVAL;
 	}
 
-	if (start >= length) {
-		log_dbg("ERROR: Missing trailing null byte beyond parsed json data string.");
+	if (json_len >= max_length) {
+		log_dbg(cd, "ERROR: Missing trailing null byte beyond parsed json data string.");
 		return -EINVAL;
 	}
 
 	/*
 	 * TODO:
 	 *	validate there are legal json format characters between
-	 *	'json_area' and 'json_area + start'
+	 *	'json_area' and 'json_area + json_len'
 	 */
 
 	do {
-		c = *(json_area + start);
+		c = *(json_area + json_len);
 		if (c != '\0') {
-			log_dbg("ERROR: Forbidden ascii code 0x%02hhx found beyond json data string at offset %d.",
-				c, start);
+			log_dbg(cd, "ERROR: Forbidden ascii code 0x%02hhx found beyond json data string at offset %" PRIu64,
+				c, json_len);
 			return -EINVAL;
 		}
-	} while (++start < length);
+	} while (++json_len < max_length);
 
 	return 0;
 }
 
-static int validate_luks2_json_object(json_object *jobj_hdr)
+static int validate_luks2_json_object(struct crypt_device *cd, json_object *jobj_hdr, uint64_t length)
 {
 	int r;
 
 	/* we require top level object to be of json_type_object */
 	r = !json_object_is_type(jobj_hdr, json_type_object);
 	if (r) {
-		log_dbg("ERROR: Resulting object is not a json object type");
+		log_dbg(cd, "ERROR: Resulting object is not a json object type");
 		return r;
 	}
 
-	r = LUKS2_hdr_validate(jobj_hdr);
+	r = LUKS2_hdr_validate(cd, jobj_hdr, length);
 	if (r) {
-		log_dbg("Repairing JSON metadata.");
+		log_dbg(cd, "Repairing JSON metadata.");
 		/* try to correct known glitches */
-		LUKS2_hdr_repair(jobj_hdr);
+		LUKS2_hdr_repair(cd, jobj_hdr);
 
 		/* run validation again */
-		r = LUKS2_hdr_validate(jobj_hdr);
+		r = LUKS2_hdr_validate(cd, jobj_hdr, length);
 	}
 
 	if (r)
-		log_dbg("ERROR: LUKS2 validation failed");
+		log_dbg(cd, "ERROR: LUKS2 validation failed");
 
 	return r;
 }
 
-static json_object *parse_and_validate_json(const char *json_area, int length)
+static json_object *parse_and_validate_json(struct crypt_device *cd,
+					    const char *json_area, uint64_t max_length)
 {
-	int offset, r;
-	json_object *jobj = parse_json_len(json_area, length, &offset);
+	int json_len, r;
+	json_object *jobj = parse_json_len(cd, json_area, max_length, &json_len);
 
 	if (!jobj)
 		return NULL;
 
 	/* successful parse_json_len must not return offset <= 0 */
-	assert(offset > 0);
+	assert(json_len > 0);
 
-	r = validate_json_area(json_area, offset, length);
+	r = validate_json_area(cd, json_area, json_len, max_length);
 	if (!r)
-		r = validate_luks2_json_object(jobj);
+		r = validate_luks2_json_object(cd, jobj, max_length);
 
 	if (r) {
 		json_object_put(jobj);
@@ -532,19 +515,19 @@ static json_object *parse_and_validate_json(const char *json_area, int length)
 	return jobj;
 }
 
-static int detect_device_signatures(const char *path)
+static int detect_device_signatures(struct crypt_device *cd, const char *path)
 {
 	blk_probe_status prb_state;
 	int r;
 	struct blkid_handle *h;
 
 	if (!blk_supported()) {
-		log_dbg("Blkid probing of device signatures disabled.");
+		log_dbg(cd, "Blkid probing of device signatures disabled.");
 		return 0;
 	}
 
 	if ((r = blk_init_by_path(&h, path))) {
-		log_dbg("Failed to initialize blkid_handle by path.");
+		log_dbg(cd, "Failed to initialize blkid_handle by path.");
 		return -EINVAL;
 	}
 
@@ -558,22 +541,22 @@ static int detect_device_signatures(const char *path)
 
 	switch (prb_state) {
 	case PRB_AMBIGUOUS:
-		log_dbg("Blkid probe couldn't decide device type unambiguously.");
+		log_dbg(cd, "Blkid probe couldn't decide device type unambiguously.");
 		/* fall through */
 	case PRB_FAIL:
-		log_dbg("Blkid probe failed.");
+		log_dbg(cd, "Blkid probe failed.");
 		r = -EINVAL;
 		break;
 	case PRB_OK: /* crypto_LUKS type is filtered out */
 		r = -EINVAL;
 
 		if (blk_is_partition(h))
-			log_dbg("Blkid probe detected partition type '%s'", blk_get_partition_type(h));
+			log_dbg(cd, "Blkid probe detected partition type '%s'", blk_get_partition_type(h));
 		else if (blk_is_superblock(h))
-			log_dbg("blkid probe detected superblock type '%s'", blk_get_superblock_type(h));
+			log_dbg(cd, "blkid probe detected superblock type '%s'", blk_get_superblock_type(h));
 		break;
 	case PRB_EMPTY:
-		log_dbg("Blkid probe detected no foreign device signature.");
+		log_dbg(cd, "Blkid probe detected no foreign device signature.");
 	}
 	blk_free(h);
 	return r;
@@ -598,16 +581,16 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 	/* Skip auto-recovery if locks are disabled and we're not doing LUKS2 explicit repair */
 	if (do_recovery && do_blkprobe && !crypt_metadata_locking_enabled()) {
 		do_recovery = 0;
-		log_dbg("Disabling header auto-recovery due to locking being disabled.");
+		log_dbg(cd, "Disabling header auto-recovery due to locking being disabled.");
 	}
 
 	/*
 	 * Read primary LUKS2 header (offset 0).
 	 */
 	state_hdr1 = HDR_FAIL;
-	r = hdr_read_disk(device, &hdr_disk1, &json_area1, 0, 0);
+	r = hdr_read_disk(cd, device, &hdr_disk1, &json_area1, 0, 0);
 	if (r == 0) {
-		jobj_hdr1 = parse_and_validate_json(json_area1, be64_to_cpu(hdr_disk1.hdr_size) - LUKS2_HDR_BIN_LEN);
+		jobj_hdr1 = parse_and_validate_json(cd, json_area1, be64_to_cpu(hdr_disk1.hdr_size) - LUKS2_HDR_BIN_LEN);
 		state_hdr1 = jobj_hdr1 ? HDR_OK : HDR_OBSOLETE;
 	} else if (r == -EIO)
 		state_hdr1 = HDR_FAIL_IO;
@@ -617,9 +600,9 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 	 */
 	state_hdr2 = HDR_FAIL;
 	if (state_hdr1 != HDR_FAIL && state_hdr1 != HDR_FAIL_IO) {
-		r = hdr_read_disk(device, &hdr_disk2, &json_area2, be64_to_cpu(hdr_disk1.hdr_size), 1);
+		r = hdr_read_disk(cd, device, &hdr_disk2, &json_area2, be64_to_cpu(hdr_disk1.hdr_size), 1);
 		if (r == 0) {
-			jobj_hdr2 = parse_and_validate_json(json_area2, be64_to_cpu(hdr_disk2.hdr_size) - LUKS2_HDR_BIN_LEN);
+			jobj_hdr2 = parse_and_validate_json(cd, json_area2, be64_to_cpu(hdr_disk2.hdr_size) - LUKS2_HDR_BIN_LEN);
 			state_hdr2 = jobj_hdr2 ? HDR_OK : HDR_OBSOLETE;
 		} else if (r == -EIO)
 			state_hdr2 = HDR_FAIL_IO;
@@ -628,10 +611,10 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 		 * No header size, check all known offsets.
 		 */
 		for (r = -EINVAL,i = 0; r < 0 && i < ARRAY_SIZE(hdr2_offsets); i++)
-			r = hdr_read_disk(device, &hdr_disk2, &json_area2, hdr2_offsets[i], 1);
+			r = hdr_read_disk(cd, device, &hdr_disk2, &json_area2, hdr2_offsets[i], 1);
 
 		if (r == 0) {
-			jobj_hdr2 = parse_and_validate_json(json_area2, be64_to_cpu(hdr_disk2.hdr_size) - LUKS2_HDR_BIN_LEN);
+			jobj_hdr2 = parse_and_validate_json(cd, json_area2, be64_to_cpu(hdr_disk2.hdr_size) - LUKS2_HDR_BIN_LEN);
 			state_hdr2 = jobj_hdr2 ? HDR_OK : HDR_OBSOLETE;
 		} else if (r == -EIO)
 			state_hdr2 = HDR_FAIL_IO;
@@ -657,7 +640,7 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 		goto err;
 	}
 
-	r = LUKS2_check_device_size(cd, device, hdr_size, 0);
+	r = device_check_size(cd, device, hdr_size, 0);
 	if (r)
 		goto err;
 
@@ -665,9 +648,9 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 	 * Try to rewrite (recover) bad header. Always regenerate salt for bad header.
 	 */
 	if (state_hdr1 == HDR_OK && state_hdr2 != HDR_OK) {
-		log_dbg("Secondary LUKS2 header requires recovery.");
+		log_dbg(cd, "Secondary LUKS2 header requires recovery.");
 
-		if (do_blkprobe && (r = detect_device_signatures(device_path(device)))) {
+		if (do_blkprobe && (r = detect_device_signatures(cd, device_path(device)))) {
 			log_err(cd, _("Device contains ambiguous signatures, cannot auto-recover LUKS2.\n"
 				      "Please run \"cryptsetup repair\" for recovery."));
 			goto err;
@@ -675,20 +658,20 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 
 		if (do_recovery) {
 			memcpy(&hdr_disk2, &hdr_disk1, LUKS2_HDR_BIN_LEN);
-			r = crypt_random_get(NULL, (char*)hdr_disk2.salt, sizeof(hdr_disk2.salt), CRYPT_RND_SALT);
+			r = crypt_random_get(cd, (char*)hdr_disk2.salt, sizeof(hdr_disk2.salt), CRYPT_RND_SALT);
 			if (r)
-				log_dbg("Cannot generate master salt.");
+				log_dbg(cd, "Cannot generate master salt.");
 			else {
 				hdr_from_disk(&hdr_disk1, &hdr_disk2, hdr, 0);
-				r = hdr_write_disk(device, hdr, json_area1, 1);
+				r = hdr_write_disk(cd, device, hdr, json_area1, 1);
 			}
 			if (r)
-				log_dbg("Secondary LUKS2 header recovery failed.");
+				log_dbg(cd, "Secondary LUKS2 header recovery failed.");
 		}
 	} else if (state_hdr1 != HDR_OK && state_hdr2 == HDR_OK) {
-		log_dbg("Primary LUKS2 header requires recovery.");
+		log_dbg(cd, "Primary LUKS2 header requires recovery.");
 
-		if (do_blkprobe && (r = detect_device_signatures(device_path(device)))) {
+		if (do_blkprobe && (r = detect_device_signatures(cd, device_path(device)))) {
 			log_err(cd, _("Device contains ambiguous signatures, cannot auto-recover LUKS2.\n"
 				      "Please run \"cryptsetup repair\" for recovery."));
 			goto err;
@@ -696,15 +679,15 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 
 		if (do_recovery) {
 			memcpy(&hdr_disk1, &hdr_disk2, LUKS2_HDR_BIN_LEN);
-			r = crypt_random_get(NULL, (char*)hdr_disk1.salt, sizeof(hdr_disk1.salt), CRYPT_RND_SALT);
+			r = crypt_random_get(cd, (char*)hdr_disk1.salt, sizeof(hdr_disk1.salt), CRYPT_RND_SALT);
 			if (r)
-				log_dbg("Cannot generate master salt.");
+				log_dbg(cd, "Cannot generate master salt.");
 			else {
 				hdr_from_disk(&hdr_disk2, &hdr_disk1, hdr, 1);
-				r = hdr_write_disk(device, hdr, json_area2, 0);
+				r = hdr_write_disk(cd, device, hdr, json_area2, 0);
 			}
 			if (r)
-				log_dbg("Primary LUKS2 header recovery failed.");
+				log_dbg(cd, "Primary LUKS2 header recovery failed.");
 		}
 	}
 
@@ -736,7 +719,7 @@ int LUKS2_disk_hdr_read(struct crypt_device *cd, struct luks2_hdr *hdr,
 	 */
 	return 0;
 err:
-	log_dbg("LUKS2 header read failed (%d).", r);
+	log_dbg(cd, "LUKS2 header read failed (%d).", r);
 
 	free(json_area1);
 	free(json_area2);
@@ -757,7 +740,7 @@ int LUKS2_hdr_version_unlocked(struct crypt_device *cd, const char *backup_file)
 
 	if (!backup_file)
 		device = crypt_metadata_device(cd);
-	else if (device_alloc(&device, backup_file) < 0)
+	else if (device_alloc(cd, &device, backup_file) < 0)
 		return 0;
 
 	if (!device)
@@ -771,7 +754,7 @@ int LUKS2_hdr_version_unlocked(struct crypt_device *cd, const char *backup_file)
 	if (devfd < 0)
 		goto err;
 
-	if ((read_lseek_blockwise(devfd, device_block_size(device),
+	if ((read_lseek_blockwise(devfd, device_block_size(cd, device),
 	     device_alignment(device), &hdr, sizeof(hdr), 0) == sizeof(hdr)) &&
 	    !memcmp(hdr.magic, LUKS2_MAGIC_1ST, LUKS2_MAGIC_L))
 		r = (int)be16_to_cpu(hdr.version);
@@ -780,7 +763,7 @@ err:
 		close(devfd);
 
 	if (backup_file)
-		device_free(device);
+		device_free(cd, device);
 
 	return r;
 }
