@@ -1,8 +1,8 @@
 /*
  * libcryptsetup - cryptsetup library, cipher benchmark
  *
- * Copyright (C) 2012-2018, Red Hat, Inc. All rights reserved.
- * Copyright (C) 2012-2018, Milan Broz
+ * Copyright (C) 2012-2019 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2012-2019 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -63,7 +63,8 @@ static int time_ms(struct timespec *start, struct timespec *end, double *ms)
 	return 0;
 }
 
-static int cipher_perf_one(struct cipher_perf *cp, char *buf,
+static int cipher_perf_one(struct crypt_device *cd,
+			   struct cipher_perf *cp, char *buf,
 			   size_t buf_size, int enc)
 {
 	struct crypt_cipher *cipher = NULL;
@@ -75,7 +76,7 @@ static int cipher_perf_one(struct cipher_perf *cp, char *buf,
 
 	r = crypt_cipher_init(&cipher, cp->name, cp->mode, cp->key, cp->key_length);
 	if (r < 0) {
-		log_dbg("Cannot initialise cipher %s, mode %s.", cp->name, cp->mode);
+		log_dbg(cd, "Cannot initialise cipher %s, mode %s.", cp->name, cp->mode);
 		return r;
 	}
 
@@ -99,7 +100,8 @@ static int cipher_perf_one(struct cipher_perf *cp, char *buf,
 
 	return r;
 }
-static int cipher_measure(struct cipher_perf *cp, char *buf,
+static int cipher_measure(struct crypt_device *cd,
+			  struct cipher_perf *cp, char *buf,
 			  size_t buf_size, int encrypt, double *ms)
 {
 	struct timespec start, end;
@@ -112,7 +114,7 @@ static int cipher_measure(struct cipher_perf *cp, char *buf,
 	if (clock_gettime(CLOCK_MONOTONIC, &start) < 0)
 		return -EINVAL;
 
-	r = cipher_perf_one(cp, buf, buf_size, encrypt);
+	r = cipher_perf_one(cd, cp, buf, buf_size, encrypt);
 	if (r < 0)
 		return r;
 
@@ -124,7 +126,7 @@ static int cipher_measure(struct cipher_perf *cp, char *buf,
 		return r;
 
 	if (*ms < CIPHER_TIME_MIN_MS) {
-		log_dbg("Measured cipher runtime (%1.6f) is too low.", *ms);
+		log_dbg(cd, "Measured cipher runtime (%1.6f) is too low.", *ms);
 		return -ERANGE;
 	}
 
@@ -138,7 +140,7 @@ static double speed_mbs(unsigned long bytes, double ms)
 	return speed / (1024 * 1024) / s;
 }
 
-static int cipher_perf(struct cipher_perf *cp,
+static int cipher_perf(struct crypt_device *cd, struct cipher_perf *cp,
 	double *encryption_mbs, double *decryption_mbs)
 {
 	double ms_enc, ms_dec, ms;
@@ -151,7 +153,7 @@ static int cipher_perf(struct cipher_perf *cp,
 	ms_enc = 0.0;
 	repeat_enc = 1;
 	while (ms_enc < 1000.0) {
-		r = cipher_measure(cp, buf, cp->buffer_size, 1, &ms);
+		r = cipher_measure(cd, cp, buf, cp->buffer_size, 1, &ms);
 		if (r < 0) {
 			free(buf);
 			return r;
@@ -163,7 +165,7 @@ static int cipher_perf(struct cipher_perf *cp,
 	ms_dec = 0.0;
 	repeat_dec = 1;
 	while (ms_dec < 1000.0) {
-		r = cipher_measure(cp, buf, cp->buffer_size, 0, &ms);
+		r = cipher_measure(cd, cp, buf, cp->buffer_size, 0, &ms);
 		if (r < 0) {
 			free(buf);
 			return r;
@@ -224,7 +226,7 @@ int crypt_benchmark(struct crypt_device *cd,
 	if ((c  = strchr(cp.mode, '-')))
 		*c = '\0';
 
-	r = cipher_perf(&cp, encryption_mbs, decryption_mbs);
+	r = cipher_perf(cd, &cp, encryption_mbs, decryption_mbs);
 out:
 	free(cp.key);
 	free(cp.iv);
@@ -253,7 +255,7 @@ int crypt_benchmark_pbkdf(struct crypt_device *cd,
 
 	kdf_opt = !strcmp(pbkdf->type, CRYPT_KDF_PBKDF2) ? pbkdf->hash : "";
 
-	log_dbg("Running %s(%s) benchmark.", pbkdf->type, kdf_opt);
+	log_dbg(cd, "Running %s(%s) benchmark.", pbkdf->type, kdf_opt);
 
 	r = crypt_pbkdf_perf(pbkdf->type, pbkdf->hash, password, password_size,
 			     salt, salt_size, volume_key_size, pbkdf->time_ms,
@@ -261,19 +263,24 @@ int crypt_benchmark_pbkdf(struct crypt_device *cd,
 			     &pbkdf->iterations, &pbkdf->max_memory_kb, progress, usrptr);
 
 	if (!r)
-		log_dbg("Benchmark returns %s(%s) %u iterations, %u memory, %u threads (for %zu-bits key).",
+		log_dbg(cd, "Benchmark returns %s(%s) %u iterations, %u memory, %u threads (for %zu-bits key).",
 			pbkdf->type, kdf_opt, pbkdf->iterations, pbkdf->max_memory_kb,
 			pbkdf->parallel_threads, volume_key_size * 8);
 	return r;
 }
 
+struct benchmark_usrptr {
+	struct crypt_device *cd;
+	struct crypt_pbkdf_type *pbkdf;
+};
+
 static int benchmark_callback(uint32_t time_ms, void *usrptr)
 {
-	struct crypt_pbkdf_type *pbkdf = usrptr;
+	struct benchmark_usrptr *u = usrptr;
 
-	log_dbg("PBKDF benchmark: memory cost = %u, iterations = %u, "
-		"threads = %u (took %u ms)", pbkdf->max_memory_kb,
-		pbkdf->iterations, pbkdf->parallel_threads, time_ms);
+	log_dbg(u->cd, "PBKDF benchmark: memory cost = %u, iterations = %u, "
+		"threads = %u (took %u ms)", u->pbkdf->max_memory_kb,
+		u->pbkdf->iterations, u->pbkdf->parallel_threads, time_ms);
 
 	return 0;
 }
@@ -293,22 +300,25 @@ int crypt_benchmark_pbkdf_internal(struct crypt_device *cd,
 	double PBKDF2_tmp;
 	uint32_t ms_tmp;
 	int r = -EINVAL;
-
-	/* Already benchmarked */
-	if (pbkdf->iterations) {
-		log_dbg("Reusing PBKDF values.");
-		return 0;
-	}
-
-	if (pbkdf->flags & CRYPT_PBKDF_NO_BENCHMARK) {
-		log_err(cd, _("PBKDF benchmark disabled but iterations not set.\n"));
-		return -EINVAL;
-	}
+	struct benchmark_usrptr u = {
+		.cd = cd,
+		.pbkdf = pbkdf
+	};
 
 	r = crypt_pbkdf_get_limits(pbkdf->type, &pbkdf_limits);
 	if (r)
 		return r;
 
+	if (pbkdf->flags & CRYPT_PBKDF_NO_BENCHMARK) {
+		if (pbkdf->iterations) {
+			log_dbg(cd, "Reusing PBKDF values (no benchmark flag is set).");
+			return 0;
+		}
+		log_err(cd, _("PBKDF benchmark disabled but iterations not set."));
+		return -EINVAL;
+	}
+
+	/* For PBKDF2 run benchmark always. Also note it depends on volume_key_size! */
 	if (!strcmp(pbkdf->type, CRYPT_KDF_PBKDF2)) {
 		/*
 		 * For PBKDF2 it is enough to run benchmark for only 1 second
@@ -320,10 +330,10 @@ int crypt_benchmark_pbkdf_internal(struct crypt_device *cd,
 		pbkdf->max_memory_kb = 0; /* N/A in PBKDF2 */
 
 		r = crypt_benchmark_pbkdf(cd, pbkdf, "foo", 3, "bar", 3,
-					volume_key_size, &benchmark_callback, pbkdf);
+					volume_key_size, &benchmark_callback, &u);
 		pbkdf->time_ms = ms_tmp;
 		if (r < 0) {
-			log_err(cd, _("Not compatible PBKDF2 options (using hash algorithm %s).\n"),
+			log_err(cd, _("Not compatible PBKDF2 options (using hash algorithm %s)."),
 				pbkdf->hash);
 			return r;
 		}
@@ -333,11 +343,17 @@ int crypt_benchmark_pbkdf_internal(struct crypt_device *cd,
 			return -EINVAL;
 		pbkdf->iterations = at_least((uint32_t)PBKDF2_tmp, pbkdf_limits.min_iterations);
 	} else {
+		/* Already benchmarked */
+		if (pbkdf->iterations) {
+			log_dbg(cd, "Reusing PBKDF values.");
+			return 0;
+		}
+
 		r = crypt_benchmark_pbkdf(cd, pbkdf, "foo", 3,
 			"0123456789abcdef0123456789abcdef", 32,
-			volume_key_size, &benchmark_callback, pbkdf);
+			volume_key_size, &benchmark_callback, &u);
 		if (r < 0)
-			log_err(cd, _("Not compatible PBKDF options.\n"));
+			log_err(cd, _("Not compatible PBKDF options."));
 	}
 
 	return r;
