@@ -1,8 +1,8 @@
 /*
  * integritysetup - setup integrity protected volumes for dm-integrity
  *
- * Copyright (C) 2017-2018, Red Hat, Inc. All rights reserved.
- * Copyright (C) 2017-2018, Milan Broz
+ * Copyright (C) 2017-2019 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2017-2019 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,6 +39,8 @@ static int opt_buffer_sectors = 0;
 
 static int opt_no_wipe = 0;
 
+static const char *opt_data_device = NULL;
+
 static const char *opt_integrity = DEFAULT_ALG_NAME;
 static const char *opt_integrity_key_file = NULL;
 static int opt_integrity_key_size = 0;
@@ -54,6 +56,8 @@ static int opt_journal_crypt_key_size = 0;
 static int opt_integrity_nojournal = 0;
 static int opt_integrity_recovery = 0;
 
+static int opt_integrity_recalculate = 0;
+
 static int opt_version_mode = 0;
 
 static const char **action_argv;
@@ -213,7 +217,7 @@ static int action_format(int arg)
 	if (r)
 		goto out;
 
-	r = crypt_init(&cd, action_argv[0]);
+	r = crypt_init_data_device(&cd, action_argv[0], opt_data_device);
 	if (r < 0)
 		goto out;
 
@@ -299,16 +303,20 @@ static int action_open(int arg)
 	if (opt_integrity_recovery)
 		activate_flags |= CRYPT_ACTIVATE_RECOVERY;
 
+	if (opt_integrity_recalculate)
+		activate_flags |= CRYPT_ACTIVATE_RECALCULATE;
+
 	r = _read_keys(&integrity_key, &params);
 	if (r)
 		goto out;
 
-	if ((r = crypt_init(&cd, action_argv[0])))
+	if ((r = crypt_init_data_device(&cd, action_argv[0], opt_data_device)))
 		goto out;
 
 	r = crypt_load(cd, CRYPT_INTEGRITY, &params);
 	if (r)
 		goto out;
+
 	r = crypt_activate_by_volume_key(cd, action_argv[1], integrity_key,
 					 opt_integrity_key_size, activate_flags);
 out:
@@ -339,7 +347,7 @@ static int action_status(int arg)
 	struct crypt_params_integrity ip = {};
 	struct crypt_device *cd = NULL;
 	char *backing_file;
-	const char *device;
+	const char *device, *metadata_device;
 	int path = 0, r = 0;
 
 	/* perhaps a path, not a dm device name */
@@ -384,12 +392,21 @@ static int action_status(int arg)
 		log_std("  tag size: %u\n", ip.tag_size);
 		log_std("  integrity: %s\n", ip.integrity ?: "(none)");
 		device = crypt_get_device_name(cd);
-		log_std("  device:  %s\n", device);
+		metadata_device = crypt_get_metadata_device_name(cd);
+		log_std("  device:  %s%s\n", device, metadata_device ? " (detached)" : "");
 		if (crypt_loop_device(device)) {
 			backing_file = crypt_loop_backing_file(device);
 			log_std("  loop:    %s\n", backing_file);
 			free(backing_file);
 		}
+		if (metadata_device) {
+			log_std("  metadata device:  %s\n", metadata_device);
+			if (crypt_loop_device(metadata_device)) {
+				backing_file = crypt_loop_backing_file(metadata_device);
+				log_std("  loop:    %s\n", backing_file);
+				free(backing_file);
+			}
+		}
 		log_std("  sector size:  %u bytes\n", crypt_get_sector_size(cd));
 		log_std("  interleave sectors: %u\n", ip.interleave_sectors);
 		log_std("  size:    %" PRIu64 " sectors\n", cad.size);
@@ -508,6 +525,8 @@ int main(int argc, const char **argv)
 		{ "progress-frequency", '\0', POPT_ARG_INT,  &opt_progress_frequency, 0, N_("Progress line update (in seconds)"), N_("secs") },
 		{ "no-wipe",            '\0', POPT_ARG_NONE, &opt_no_wipe,            0, N_("Do not wipe device after format"), NULL },
 
+		{ "data-device",        '\0', POPT_ARG_STRING, &opt_data_device,      0, N_("Path to data device (if separated)"), N_("path") },
+
 		{ "journal-size",        'j', POPT_ARG_STRING,&opt_journal_size_str,  0, N_("Journal size"), N_("bytes") },
 		{ "interleave-sectors", '\0', POPT_ARG_INT,  &opt_interleave_sectors, 0, N_("Interleave sectors"), N_("SECTORS") },
 		{ "journal-watermark",  '\0', POPT_ARG_INT,  &opt_journal_watermark,  0, N_("Journal watermark"),N_("percent") },
@@ -530,6 +549,7 @@ int main(int argc, const char **argv)
 
 		{ "integrity-no-journal",       'D', POPT_ARG_NONE,  &opt_integrity_nojournal, 0, N_("Disable journal for integrity device"), NULL },
 		{ "integrity-recovery-mode",    'R', POPT_ARG_NONE,  &opt_integrity_recovery,  0, N_("Recovery mode (no journal, no tag checking)"), NULL },
+		{ "integrity-recalculate",     '\0', POPT_ARG_NONE,  &opt_integrity_recalculate,  0, N_("Recalculate initial tags automatically."), NULL },
 		POPT_TABLEEND
 	};
 	poptContext popt_context;
@@ -606,6 +626,11 @@ int main(int argc, const char **argv)
 	if (!strcmp(aname, "format") && opt_tag_size == 0)
 		opt_tag_size = DEFAULT_TAG_SIZE;
 
+	if (opt_integrity_recalculate && strcmp(aname, "open"))
+		usage(popt_context, EXIT_FAILURE,
+		      _("Option --integrity-recalculate can be used only for open action."),
+		      poptGetInvocationName(popt_context));
+
 	if (opt_interleave_sectors < 0 || opt_journal_watermark < 0 ||
 	    opt_journal_commit_time < 0 || opt_tag_size < 0 ||
 	    opt_sector_size < 0 || opt_buffer_sectors < 0 ||
